package it.arnaldo.unibs.tamagolem;

import java.util.*;

/**
 * Rappresenta il grafo degli elementi del gioco, dove i nodi sono {@link GraphElement}
 * e gli archi rappresentano interazioni (positive o negative) tra elementi.
 */
public class ElementGraph {

    /** Insieme degli elementi (nodi) presenti nel grafo */
    private final Set<GraphElement> elements;

    /**
     * Costruttore della classe ElementGraph.
     * Inizializza un nuovo grafo vuoto.
     */
    public ElementGraph() {
        this.elements = new HashSet<>();
    }

    /**
     * Aggiunge un nuovo elemento al grafo.
     *
     * @param element L'elemento da aggiungere
     */
    public void addElement(GraphElement element) {
        elements.add(element);
    }

    /**
     * Restituisce il valore dell'interazione tra due elementi.
     *
     * @param fromElement L'elemento di partenza
     * @param toElement L'elemento di arrivo
     * @return Il valore dell'interazione
     */
    public int getValue(Element fromElement, Element toElement) {
        for (GraphElement graphElement : elements) {
            if (graphElement.getElement().equals(fromElement)) {
                return graphElement.getLinkValue(toElement);
            }
        }
        return 0;
    }

    /**
     * Restituisce l'insieme di tutti gli elementi presenti nel grafo.
     *
     * @return Set di {@link GraphElement}
     */
    public Set<GraphElement> getElements() {
        return elements;
    }

    /**
     * Stampa a schermo la matrice delle interazioni tra tutti gli elementi del grafo.
     */
    public void printGraph() {
        System.out.println("FROM element is the row name, TO element is the column name");
        List<Element> elements = getElementsNames(); // lista ordinata degli elementi
        int maxNameLen = 0;
        for (Element element : elements) {
            if (element.name().length() > maxNameLen) {
                maxNameLen = element.name().length();
            }
        }

        maxNameLen = maxNameLen + 2;

        // Stampa intestazione colonna
        System.out.print(String.format("%" + maxNameLen + "s", ""));
        for (Element to : elements) {
            System.out.print(String.format("%" + maxNameLen + "s", to));
        }
        System.out.println();
        System.out.println("-".repeat(10 + maxNameLen * elements.size()));

        // Stampa ogni riga
        for (Element from : elements) {
            System.out.print(String.format("%" + maxNameLen + "s", from));
            for (Element to : elements) {
                int value = getInteractionBetween(from, to);
                System.out.print(String.format("%" + maxNameLen + "s", value));
            }
            System.out.println();
        }


    }

    /**
     * Calcola la forza totale di tutte le interazioni del grafo.
     *
     * @return La somma dei valori assoluti di tutte le interazioni tra elementi
     */
    public int getTotalStrength(){
        int totalStrength = 0;

        for (GraphElement element : elements) {
            totalStrength += element.getTotalStrength();
        }

        return totalStrength;
    }

    /**
     * Restituisce una lista ordinata degli elementi contenuti nel grafo.
     *
     * @return Lista di oggetti {@link Element}
     */
    public List<Element> getElementsNames(){
        List<Element> elements = new ArrayList<>();
        for (GraphElement el : this.elements ){
            elements.add(el.getElement());
        }
        return elements;
    }

    /**
     * Restituisce il valore di interazione tra due elementi.
     * Metodo alternativo a {@link #getValue}, ma lavora direttamente con i link.
     *
     * @param fromElement L'elemento di partenza
     * @param toElement L'elemento di destinazione
     * @return Il valore dell'interazione
     */
    public int getInteractionBetween(Element fromElement, Element toElement) {
        for (GraphElement el : elements) {
            if (el.getElement().equals(fromElement)){
                for (ElementLink link : el.getLinks()) {
                    if (link.getElement().equals(toElement)){
                        return link.getValue();
                    }
                }
            }
        }

        return 0;
    }


}
