package it.arnaldo.unibs.tamagolem;

import java.util.HashSet;
import java.util.Set;

/**
 * Rappresenta un nodo (elemento) nel grafo degli elementi, con i relativi collegamenti ad altri elementi.
 * Ogni collegamento è rappresentato da un oggetto {@link ElementLink}.
 */
public class GraphElement {

    /** L'elemento associato a questo nodo del grafo */
    private final Element element;

    /** Insieme dei collegamenti ad altri elementi */
    private final Set<ElementLink> links;

    /**
     * Costruttore della classe GraphElement.
     *
     * @param element L'elemento rappresentato da questo nodo
     */
    public GraphElement(Element element) {
        this.element = element;
        this.links = new HashSet<>();
    }

    /**
     * Restituisce l'elemento associato a questo nodo.
     *
     * @return L'elemento di tipo {@link Element}
     */
    public Element getElement() {
        return element;
    }

    /**
     * Restituisce l'insieme dei collegamenti a elementi adiacenti.
     *
     * @return Un insieme di oggetti {@link ElementLink}
     */
    public Set<ElementLink> getLinks() {
        return links;
    }

    /**
     * Aggiunge un collegamento (link) a questo nodo del grafo.
     *
     * @param link Il collegamento da aggiungere
     */
    public void addLink(ElementLink link) {
        links.add(link);
    }

    /**
     * Restituisce il valore del collegamento con un altro elemento.
     *
     * @param element L'elemento adiacente di cui cercare il collegamento
     * @return Il valore associato al collegamento
     */
    public int getLinkValue(Element element) {
        for (ElementLink link : links) {
            if (link.getElement().equals(element)) {
                return link.getValue();
            }
        }
        return 0;
    }

    /**
     * Calcola la forza totale di tutti i collegamenti dell'elemento.
     *
     * @return La somma delle forze di tutti i collegamenti
     */
    public int getTotalStrength(){
        int strength = 0;
        for (ElementLink link : links) {
            strength += link.getTotalStrength();
        }
        return strength;
    }
}
