package it.arnaldo.unibs.tamagolem;

import it.kibo.fp.lib.RandomDraws;

import java.util.*;

/**
 * Rappresenta un sistema lineare di equazioni intere utilizzato per determinare le interazioni tra elementi.
 * Risolve il sistema tramite eliminazione di Gauss e genera soluzioni intere positive casuali.
 */
public class LinearSystem {

    /** Numero di equazioni */
    private final int n;

    /** Numero di incognite (pari a n*(n-1)/2) */
    private final int m;

    /** Matrice dei coefficienti (dimensione n x m) */
    private final int[][] A;

    /** Vettore dei termini noti */
    private final int[] b;

    /**
     * Costruttore della classe LinearSystem.
     *
     * @param n Numero di equazioni
     * @param A Matrice dei coefficienti
     * @param b Vettore dei termini noti
     */
    public LinearSystem(int n, int[][] A, int[] b) {
        this.n = n;
        this.m = (n * (n - 1)) / 2;
        this.A = A;
        this.b = b;
    }

    /**
     * Risolve il sistema lineare cercando una soluzione intera positiva non nulla.
     *
     * @return Un array di interi con la soluzione del sistema
     * @throws IllegalStateException Se non esiste una soluzione intera positiva valida
     */
    public int[] resolve() {
        int[][] extendedMatrix = new int[n][m + 1];

        for (int i = 0; i < n; i++) {
            System.arraycopy(A[i], 0, extendedMatrix[i], 0, m);
            extendedMatrix[i][m] = b[i];
        }

        int rank = gauss(extendedMatrix);
        int indipendentVariables = m - rank;
        if (indipendentVariables <= 0) {
            throw new IllegalStateException("Sistema determinato o sovradeterminato");
        }

        int[] solutions;
        int tries = 1000000;
        while (tries-- > 0) {
            int[] lambdaValues = new int[indipendentVariables];
            for (int i = 0; i < indipendentVariables; i++) {
                lambdaValues[i] = RandomDraws.drawInteger(1, m);
            }

            solutions = constructSolution(extendedMatrix, lambdaValues, rank);
            if (validNumbers(solutions)) return solutions;
        }

        throw new IllegalStateException("Impossibile trovare una solutions intera senza zeri");
    }

    /**
     * Verifica che tutti i numeri della soluzione siano maggiori di zero.
     *
     * @param x Soluzione da verificare
     * @return true se tutti i valori sono positivi, false altrimenti
     */
    private boolean validNumbers(int[] x) {
        for (int xi : x) {
            if (xi <= 0) return false;
        }

        return true;
    }

    /**
     * Costruisce una soluzione completa del sistema a partire da parametri liberi e matrice estesa.
     *
     * @param extendedMatrix La matrice aumentata del sistema
     * @param parameters Valori delle variabili libere
     * @param rank Grado del sistema
     * @return Un array contenente la soluzione del sistema
     */
    private int[] constructSolution(int[][] extendedMatrix, int[] parameters, int rank) {
        int[] x = new int[m];
        Arrays.fill(x, 0);

        List<Integer> pivotCol = new ArrayList<>();
        boolean[] isFree = new boolean[m];
        Arrays.fill(isFree, true);

        for (int i = 0; i < rank; i++) {
            int col = 0;
            while (col < m && extendedMatrix[i][col] == 0) col++;
            pivotCol.add(col);
            isFree[col] = false;
        }

        int paramIndex = 0;
        for (int i = 0; i < m; i++) {
            if (isFree[i]) {
                x[i] = parameters[paramIndex++];
            }
        }

        for (int i = rank - 1; i >= 0; i--) {
            int pivot = pivotCol.get(i);
            int val = extendedMatrix[i][m];
            for (int j = pivot + 1; j < m; j++) {
                val -= extendedMatrix[i][j] * x[j];
            }

            if (val % extendedMatrix[i][pivot] != 0) {
                return new int[m]; // soluzione non intera -> rifiutiamo
            }

            x[pivot] = val / extendedMatrix[i][pivot];
        }

        return x;
    }

    /**
     * Applica l'eliminazione di Gauss per determinare il rango della matrice.
     *
     * @param mat Matrice aumentata da trasformare
     * @return Il rango della matrice
     */
    private int gauss(int[][] mat) {
        int row = 0;
        for (int col = 0; col < m && row < n; col++) {
            int pivot = row;
            for (int i = row + 1; i < n; i++) {
                if (Math.abs(mat[i][col]) > Math.abs(mat[pivot][col])) {
                    pivot = i;
                }
            }

            if (mat[pivot][col] == 0) continue;

            int[] temp = mat[row];
            mat[row] = mat[pivot];
            mat[pivot] = temp;

            for (int i = row + 1; i < n; i++) {
                int a = mat[i][col];
                int b = mat[row][col];
                if (a == 0) continue;

                int mcm = mcm(Math.abs(a), Math.abs(b));
                int f1 = mcm / Math.abs(a);
                int f2 = mcm / Math.abs(b);
                if (a * b < 0) f2 = -f2;

                for (int j = col; j <= m; j++) {
                    mat[i][j] = mat[i][j] * f1 - mat[row][j] * f2;
                }
            }

            row++;
        }

        return row;
    }

    /**
     * Calcola il Massimo Comun Divisore (MCD) tra due numeri.
     *
     * @param a Primo numero
     * @param b Secondo numero
     * @return Il MCD
     */
    private int mcd(int a, int b) {
        return b == 0 ? a : mcd(b, a % b);
    }

    /**
     * Calcola il Minimo Comune Multiplo (MCM) tra due numeri.
     *
     * @param a Primo numero
     * @param b Secondo numero
     * @return Il MCM
     */
    private int mcm(int a, int b) {
        return a / mcd(a, b) * b;
    }
}
