package it.arnaldo.unibs.tamagolem;

import java.util.ArrayDeque;
import java.util.Deque;

/**
 * Rappresenta un TamaGolem, ovvero una creatura che combatte usando pietre elementali.
 * Ogni TamaGolem ha dei punti vita e una coda di pietre che utilizza ciclicamente in battaglia.
 */
public class TamaGolem {

    /** Coda di pietre elementali che il TamaGolem utilizza nei combattimenti */
    private final Deque<Element> elementalStones;

    /** Punti vita del TamaGolem */
    private int lifePoints;

    /**
     * Costruttore della classe TamaGolem.
     *
     * @param lifePoints I punti vita iniziali del TamaGolem
     */
    public TamaGolem(int lifePoints) {
        this.lifePoints = lifePoints;
        this.elementalStones = new ArrayDeque<>();
    }

    /**
     * Restituisce la coda di pietre elementali del TamaGolem.
     *
     * @return Una Deque contenente le pietre elementali
     */
    public Deque<Element> getElementalStones() {
        return elementalStones;
    }

    /**
     * Aggiunge una pietra elementale alla coda del TamaGolem.
     *
     * @param stone La pietra da aggiungere
     */
    public void addElementalStone(Element stone) {
        elementalStones.add(stone);
    }

    /**
     * Utilizza la prima pietra elementale nella coda e la reinserisce in fondo,
     * simulando un utilizzo ciclico.
     *
     * @return La pietra utilizzata, oppure null se la coda è vuota
     */
    public Element useElementalStone() {
        if (!elementalStones.isEmpty()) {
            Element removedStone = elementalStones.removeFirst();
            elementalStones.add(removedStone);
            return removedStone;
        }
        return null;
    }

    /**
     * Restituisce i punti vita attuali del TamaGolem.
     *
     * @return I punti vita rimanenti
     */
    public int getLifePoints() {
        return lifePoints;
    }

    /**
     * Applica un danno al TamaGolem, riducendone i punti vita.
     *
     * @param damage Quantità di danno da applicare
     * @return true se il TamaGolem è morto (punti vita ≤ 0), false altrimenti
     */
    public boolean getDamage(int damage) {
        lifePoints -= damage;
        return lifePoints <= 0;
    }

    /**
     * Verifica se il TamaGolem è ancora vivo.
     *
     * @return true se i punti vita sono maggiori di zero, false altrimenti
     */
    public boolean isAlive() {
        return lifePoints > 0;
    }

    /**
     * Restituisce una rappresentazione testuale dello stato del TamaGolem.
     *
     * @return Una stringa con i punti vita e le pietre elementali
     */
    @Override
    public String toString() {
        return "LP: " + lifePoints + ", Elemental stones:" + elementalStones.toString();
    }

    /**
     * Stampa a console lo stato corrente del TamaGolem.
     */
    public void print() {
        System.out.println(toString());

    }
}
